@@ -31,6 +31,8 @@
 #include "disk-io.h"
 #include "commands.h"
 #include "btrfs-list.h"
+#include "cmds-inspect-dump-tree.h"
+#include "cmds-inspect-dump-super.h"
 
 static const char * const inspect_cmd_group_usage[] = {
 	"btrfs inspect-internal <command> <args>",
@@ -42,20 +44,16 @@ static int __ino_to_path_fd(u64 inum, int fd, int verbose, const char *prepend)
 	int ret;
 	int i;
 	struct btrfs_ioctl_ino_path_args ipa;
-	struct btrfs_data_container *fspath;
-
-	fspath = malloc(4096);
-	if (!fspath)
-		return -ENOMEM;
+	struct btrfs_data_container fspath[PATH_MAX];
 
 	memset(fspath, 0, sizeof(*fspath));
 	ipa.inum = inum;
-	ipa.size = 4096;
+	ipa.size = PATH_MAX;
 	ipa.fspath = ptr_to_u64(fspath);
 
 	ret = ioctl(fd, BTRFS_IOC_INO_PATHS, &ipa);
-	if (ret) {
-		printf("ioctl ret=%d, error: %s\n", ret, strerror(errno));
+	if (ret < 0) {
+		error("ino paths ioctl: %s", strerror(errno));
 		goto out;
 	}
 
@@ -79,7 +77,6 @@ static int __ino_to_path_fd(u64 inum, int fd, int verbose, const char *prepend)
 	}
 
 out:
-	free(fspath);
 	return !!ret;
 }
 
@@ -194,8 +191,8 @@ static int cmd_inspect_logical_resolve(int argc, char **argv)
 	}
 
 	ret = ioctl(fd, BTRFS_IOC_LOGICAL_INO, &loi);
-	if (ret) {
-		printf("ioctl ret=%d, error: %s\n", ret, strerror(errno));
+	if (ret < 0) {
+		error("logical ino ioctl: %s", strerror(errno));
 		goto out;
 	}
 
@@ -271,22 +268,23 @@ static int cmd_inspect_subvolid_resolve(int argc, char **argv)
 	char path[PATH_MAX];
 	DIR *dirstream = NULL;
 
-	if (check_argc_exact(argc, 3))
+	clean_args_no_options(argc, argv, cmd_inspect_subvolid_resolve_usage);
+
+	if (check_argc_exact(argc - optind, 2))
 		usage(cmd_inspect_subvolid_resolve_usage);
 
-	fd = btrfs_open_dir(argv[2], &dirstream, 1);
+	fd = btrfs_open_dir(argv[optind + 1], &dirstream, 1);
 	if (fd < 0) {
 		ret = -ENOENT;
 		goto out;
 	}
 
-	subvol_id = arg_strtou64(argv[1]);
+	subvol_id = arg_strtou64(argv[optind]);
 	ret = btrfs_subvolid_resolve(fd, path, sizeof(path), subvol_id);
 
 	if (ret) {
-		fprintf(stderr,
-			"%s: btrfs_subvolid_resolve(subvol_id %llu) failed with ret=%d\n",
-			argv[0], (unsigned long long)subvol_id, ret);
+		error("resolving subvolid %llu error %d",
+			(unsigned long long)subvol_id, ret);
 		goto out;
 	}
 
@@ -311,10 +309,12 @@ static int cmd_inspect_rootid(int argc, char **argv)
 	u64 rootid;
 	DIR *dirstream = NULL;
 
-	if (check_argc_exact(argc, 2))
+	clean_args_no_options(argc, argv, cmd_inspect_rootid_usage);
+
+	if (check_argc_exact(argc - optind, 1))
 		usage(cmd_inspect_rootid_usage);
 
-	fd = btrfs_open_dir(argv[1], &dirstream, 1);
+	fd = btrfs_open_dir(argv[optind], &dirstream, 1);
 	if (fd < 0) {
 		ret = -ENOENT;
 		goto out;
@@ -322,8 +322,7 @@ static int cmd_inspect_rootid(int argc, char **argv)
 
 	ret = lookup_ino_rootid(fd, &rootid);
 	if (ret) {
-		fprintf(stderr, "%s: rootid failed with ret=%d\n",
-			argv[0], ret);
+		error("rootid failed with ret=%d", ret);
 		goto out;
 	}
 
@@ -334,6 +333,14 @@ out:
 	return !!ret;
 }
 
+static const char* const cmd_inspect_min_dev_size_usage[] = {
+	"btrfs inspect-internal min-dev-size [options] <path>",
+	"Get the minimum size the device can be shrunk to. The",
+	"device id 1 is used by default.",
+	"--id DEVID   specify the device id to query",
+	NULL
+};
+
 struct dev_extent_elem {
 	u64 start;
 	/* inclusive end */
@@ -513,9 +520,7 @@ static int print_min_dev_size(int fd, u64 devid)
 
 		ret = ioctl(fd, BTRFS_IOC_TREE_SEARCH, &args);
 		if (ret < 0) {
-			fprintf(stderr,
-				"Error invoking tree search ioctl: %s\n",
-				strerror(errno));
+			error("tree search ioctl: %s", strerror(errno));
 			ret = 1;
 			goto out;
 		}
@@ -551,7 +556,7 @@ static int print_min_dev_size(int fd, u64 devid)
 				ret = add_dev_extent(&holes, last_pos,
 						     sh->offset - 1, 1);
 			if (ret) {
-				fprintf(stderr, "Error: %s\n", strerror(-ret));
+				error("add device extent: %s", strerror(-ret));
 				ret = 1;
 				goto out;
 			}
@@ -574,14 +579,6 @@ out:
 	return ret;
 }
 
-static const char* const cmd_inspect_min_dev_size_usage[] = {
-	"btrfs inspect-internal min-dev-size [options] <path>",
-	"Get the minimum size the device can be shrunk to. The",
-	"device id 1 is used by default.",
-	"--id DEVID   specify the device id to query",
-	NULL
-};
-
 static int cmd_inspect_min_dev_size(int argc, char **argv)
 {
 	int ret;
@@ -639,6 +636,10 @@ const struct cmd_group inspect_cmd_group = {
 			0 },
 		{ "min-dev-size", cmd_inspect_min_dev_size,
 			cmd_inspect_min_dev_size_usage, NULL, 0 },
+		{ "dump-tree", cmd_inspect_dump_tree,
+				cmd_inspect_dump_tree_usage, NULL, 0 },
+		{ "dump-super", cmd_inspect_dump_super,
+				cmd_inspect_dump_super_usage, NULL, 0 },
 		NULL_CMD_STRUCT
 	}
 };
